local volume = gkini.ReadInt("Countdown", "volume", 100)/100
local gmt_offset
do
   local now = os.time()
   gmt_offset = os.time(os.date("!*t",now)) - now
   if(os.date("*t").isdst) then gmt_offset = gmt_offset - 3600 end
end
local loaded_sounds = {}
local last_played = {}
local play_delay = 10
local function play_sound(format, ...)
   local path = ("plugins/countdown/%s"):format(format:format(...))
   if(not loaded_sounds[path]) then
      loaded_sounds[path] = gksound.GKLoadSound{soundname=path, filename=path}
      if(not loaded_sounds[path]) then return end
   end
   if(last_played[path] and last_played[path] + 10 > os.time()) then return end
   last_played[path] = os.time()
   gksound.GKPlaySound(path, volume)
end

local countdown_parts = {
   {timeout=300,function() play_sound("announcer/5_minute_warning.ogg") end},
   {timeout=180,function() play_sound("announcer/3_minute_warning.ogg") end},
   {timeout=120,function() play_sound("announcer/2_minute_warning.ogg") end},
   {timeout= 60,function() play_sound("announcer/1_minute_warning.ogg") end},
   {timeout= 30,function() play_sound("announcer/30_second_warning.ogg") end},
   {timeout= 20,function() play_sound("announcer/20_second_warning.ogg") end},
   {timeout= 10,function() play_sound("announcer/10.ogg") end},
   {timeout=  9,function() play_sound("announcer/9.ogg") end},
   {timeout=  8,function() play_sound("announcer/8.ogg") end},
   {timeout=  7,function() play_sound("announcer/7.ogg") end},
   {timeout=  6,function() play_sound("announcer/6.ogg") end},
   {timeout=  5,function() play_sound("announcer/5.ogg") end},
   {timeout=  4,function() play_sound("announcer/4.ogg") end},
   {timeout=  3,function() play_sound("announcer/3.ogg") end},
   {timeout=  2,function() play_sound("announcer/2.ogg") end},
   {timeout=  1,function() play_sound("announcer/1.ogg") end},
   {timeout=  0,function() play_sound("297gq_abbrev.ogg") end},
}

local teamtimers = {}
local function countdown(team, rem, custom_finish)
   local tt = {}
   rem = math.floor(rem or 300)
   teamtimers[team] = tt
   for i,v in ipairs(countdown_parts) do
      local func = v[1]
      if(v.timeout == 0 and custom_finish) then func = custom_finish end
      if(rem > v.timeout) then
	 tt[i] = Timer()
	 tt[i]:SetTimeout(1000*(rem-v.timeout), func)
      elseif (rem == v.timeout) then
	 func()
      end
   end
end

local matches = {}
matches[201] = {}
matches[202] = {}
matches[201]["^Itani convoy scheduled to depart .+ in five minutes%.$"] = function()
   countdown("blue")
end
matches[201]["has been destroyed in (.+)!$"] = function(place)
   if(place == "Edras System, Sector I-2" or place:match("^Jallik") or place:match("^Divinia")) then
      play_sound("announcer/last_second_save.ogg")
   else
      play_sound("announcer/blue_destroyed.ogg")
   end
end
matches[201]["under attack in .+!$"] = function()
   play_sound("announcer/blue_under_attack.ogg")
end
matches[201]["delivered ([0-9]+) units? to"] = function()
   play_sound("announcer/blue_score.ogg")
end
matches[202]["^Serco convoy scheduled to depart .+ in five minutes%.$"] = function()
   countdown("red")
end
matches[202]["has been destroyed in (.+)!$"] = function(place)
   if(place == "Ukari System, Sector A-10" or place:match("^Pyronis") or place:match("^Initros")) then
      play_sound("announcer/last_second_save.ogg")
   else
      play_sound("announcer/red_destroyed.ogg")
   end
end
matches[202]["under attack in .+!$"] = function()
   play_sound("announcer/red_under_attack.ogg")
end
matches[202]["delivered ([0-9]+) units? to"] = function()
   play_sound("announcer/red_score.ogg")
end

local function OnEvent(event, arg)
   local matches = matches[arg.channelid]
   if(matches) then
      for k,v in pairs(matches) do
	 local matched = {arg.msg:match(k)}
	 if(matched[1]) then
	    v(unpack(matched))
	 end
      end
   end
end

local countdown_commands = {}

function countdown_commands.ctc(onoff)
   if(onoff == "on") then
      RegisterEvent(OnEvent, "CHAT_MSG_SERVER_CHANNEL")
      RegisterEvent(OnEvent, "CHAT_MSG_SERVER_CHANNEL_ACTIVE")
      gkini.WriteInt("Countdown", "ctc_enabled", 1)
   elseif (onoff == "off") then
      UnregisterEvent(OnEvent, "CHAT_MSG_SERVER_CHANNEL")
      UnregisterEvent(OnEvent, "CHAT_MSG_SERVER_CHANNEL_ACTIVE")
      gkini.WriteInt("Countdown", "ctc_enabled", 0)
   else
      print("You must specify 'on' or 'off'")
   end
end
function countdown_commands.volume(myvolume)
   local vol = tonumber(myvolume)
   if(not vol or vol < 0 or vol > 1) then print("That's a silly volume.") return end
   gkini.WriteInt("Countdown", "volume", math.ceil(vol * 100))
   volume = vol
end
function countdown_commands.start(mytime, command)
   local time
   if(mytime:sub(-1,-1) == "m") then
      time = tonumber(mytime:sub(1,-2))
      if(not time or time <= 0) then print("That's a silly number of minutes.") return end
      time = time * 60
   elseif (mytime:match("^[0-2]?[0-9]:[0-5]?[0-9]:[0-5]?[0-9]$")) then
      local h,m,s = assert(mytime:match("^([0-2]?[0-9]):([0-5]?[0-9]):([0-5]?[0-9])$"))
      h,m,s = tonumber(h),tonumber(m),tonumber(s)
      local now = os.time() + gmt_offset
      local newt = os.date("*t", now)
      if(newt.hour > h or
	 (newt.hour == h and (newt.min > m or (newt.min == m and newt.sec > s)))) then
	 newt.day = newt.day + 1
      end
      newt.hour,newt.min,newt.sec = h,m,s
      time = os.time(newt) - now
      assert(time >= 0)
   else
      time = tonumber(mytime)
      if(not time or time <= 0) then print("That's a silly number of seconds.") return end
   end
   if(time >= 60) then
      print("\127ffffffCountdown started. Will end at "..os.date("!%c", os.time()+time)..", unless the interface is reset.")
   end
   if(command) then
      countdown("custom", time, function() gkinterface.GKProcessCommand(command) end)
   else
      countdown("custom", time)
   end
end
countdown_commands.alarm = countdown_parts[#countdown_parts][1]

local function countdown(useless, arg)
   arg = arg or {}
   if(not arg[1] or not countdown_commands[arg[1]:lower()]) then
      print("\127ffffffUsage:\n/countdown volume 0-1 \127afafaf(changes are persistent)\127ffffff\n/countdown start <seconds> [command]\n/countdown start <minutes>m [command]\n/countdown start <HH:MM:SS> [command] \127afafaf(time is in GMT)\127ffffff\n/countdown ctc <on/off> \127afafaf(off does not disable countdowns already started, defaults to on, changes are persistent)\127ffffff\n/countdown alarm \127afafaf(welcome to the USS Ling!)")
      return
   end
   return countdown_commands[arg[1]:lower()](select(2,unpack(arg)))
end

RegisterUserCommand("countdown", countdown)
if(gkini.ReadInt("Countdown", "ctc_enabled", 1) ~= 0) then
   countdown("countdown", {"ctc", "on"})
end
